%define ALD_Ver		1.2.1

%define OrgName binutils

License:      GPL
Summary:      GNU binutils
Group:        Development/Tools
Version:      2.22
Release:      3

Source: %{OrgName}-%{version}.tar.xz

%define DestDir		/Compiler

# -----------------------------------------------------
%if %{_target_cpu} == %{_build_cpu}
%define _CROSS 0
%define BUILD_LOCAL_CROSS 0
%else
# rpm --target=x86_64 -ba xxxxx.spec
%define _CROSS 1
%define BUILD_LOCAL_CROSS 1
%endif

%define TO_ARCH %{_target_cpu}

%if %{_target_cpu} == "armv6l"
%define ARM_ARCH armv6zk
%define ARM_FPU  -mfpu=vfp -mfloat-abi=hard
%define TO_ARCH %{ARM_ARCH}
%endif

%if %{_target_cpu} == "armv7l"
%define ARM_ARCH armv7-a
%define ARM_FPU  -mfpu=vfpv3-d16 -mfloat-abi=hard
%define TO_ARCH %{ARM_ARCH}
%endif

%define BUILD_TAR 1
# -----------------------------------------------------
%if %{BUILD_LOCAL_CROSS}
Name:         %{OrgName}-%{TO_ARCH}
%else
Name:         %{OrgName}
%endif

#------------------ CROSS DIR --------------------
%if %{_CROSS}
%define build	%{_build}

%if %{_os} == "mingw"
%define target %{_arch}-%{_vendor}-%{_os}%{__isa_bits}
%else  #win32 || win64
%define target %{_arch}-%{_os}%{_gnu}
%endif

%define crossdir	/cross
%if %{_arch} == "arm"
%define csysdir		%{crossdir}/%{ARM_ARCH}
%else
%define csysdir		%{crossdir}/%{_target_cpu}
%endif
%define csysbindir	%{csysdir}/bin

%if %{BUILD_LOCAL_CROSS}
%define host	%{build}

%if (%{_build_cpu} == "x86_64")
%define _lib		lib64
%else
%define _lib		lib
%endif

%if %{_arch} == "x86_64"
%define _tlib		lib64
%else
%define _tlib		lib
%endif

%define CrossDestDir	/Cross
%define tsysdir		%{csysdir}
%define _prefix		%{csysdir}/usr
%define _bindir		%{csysdir}/bin
%define _sbindir	%{csysdir}/sbin
%define _libdir		%{csysdir}/usr/%{_lib}
%define _exec_prefix	%{_libdir}/gcc
%define _libexecdir	%{_libdir}

%define bt_prefix	%{_exec_prefix}/%{_build}/%{target}
%define bt_includedir	%{bt_prefix}/include
%define bt_libdir	%{bt_prefix}/%{_lib}

%else  #BUILD_LOCAL_CROSS
%define host	%{target}
%define _tlib		%{_lib}

%define CrossDestDir	%{nil}
%define tsysdir		%{nil}
%define _bindir		%{_prefix}/bin
%define _sbindir	%{_prefix}/sbin
%define _libdir		%{_prefix}/%{_lib}
%define _exec_prefix	%{_libdir}/gcc/%{_build}
%define _libexecdir	%{_libdir}

%define bt_includedir	%{_includedir}
%define bt_libdir	%{_libdir}

%define __os_install_post \
    %{csysbindir}/rpm/brp-strip \
    %{csysbindir}/rpm/brp-strip-static-archive \
    %{csysbindir}/rpm/brp-strip-shared \
    %{csysbindir}/rpm/brp-strip-comment-note \
    %{_rpmconfigdir}/brp-compress \
%{nil}
%endif #!BUILD_LOCAL_CROSS

%define _srcrpmdir %{_topdir}/SRPMS/tmp

%else  #_CROSS

%define build	%{_arch}-%{_os}%{_gnu}
%define target	%{build}
%define host	%{build}

%define csysdir		%{nil}
%define csysbindir	%{_prefix}/bin
%define CrossDestDir	%{nil}
%define tsysdir		%{nil}
%define _srcrpmdir	%{_topdir}/SRPMS/%{ALD_Ver}

%define _bindir		%{_prefix}/bin
%define _sbindir	%{_prefix}/sbin
%define _libdir		%{_prefix}/%{_lib}
%define _exec_prefix	%{_libdir}/gcc
%define _libexecdir	%{_libdir}

%define bt_includedir	%{_includedir}
%define bt_libdir	%{_libdir}
%define _tlib		%{_lib}

%endif # !_CROSS

%define docdir %{_defaultdocdir}/%{OrgName}
%define t_prefix	%{_exec_prefix}/%{target}

# ------------------- description ------------------------------------
%description
C compiler utilities: ar, as, gasp, gprof, nm, objcopy, objdump, ranlib,
size, strings, strip.
These utilities are needed whenever you want to compile a program.
# -------------------
%package devel
Summary:      Libraries for the C compiler
Group:        Development/Libraries
%description devel
These libraries are needed to compile C code.
# --------------------------------------
%package ldscripts
Summary:      %{name} ldscripts
Group:        Development/Libraries
%description ldscripts
%{name} ldscripts
#-------------------------------
%package info
Summary:      Info for %{name}
Group:        Documentation
%description info
Info for %{name}

# ------------------- setup ------------------------------------------
%Prep
%setup -q -n %{OrgName}-%{version}

# ------------------- Build ------------------------------------------
%Build
%if %{_CROSS}
PATH=%{csysbindir}:$PATH:/opt/perl/bin
GCC_NoHostPath=d
export PATH GCC_NoHostPath
%else
export PATH=$PATH:/opt/perl/bin
%endif

%if %{BUILD_LOCAL_CROSS}
%if %{_build_cpu} == "i386"
OPTFLAGS="-O2 -march=i486 -mpreferred-stack-boundary=2 -falign-functions=1 -falign-jumps=1 -falign-loops=1 -fomit-frame-pointer -fno-stack-protector"
%endif
%if %{_build_cpu} == "x86_64"
OPTFLAGS="-O2 -fno-asynchronous-unwind-tables"
%endif
%if (%{_build_cpu} == "armv6l") || (%{_build_cpu} == "armv7l")
OPTFLAGS="-O2 -march=%{ARM_ARCH} %{ARM_FPU} -munaligned-access"
%endif
%else
%if %{_arch} == "i386"
OPTFLAGS="-O2 -march=i486 -mpreferred-stack-boundary=2 -falign-functions=1 -falign-jumps=1 -falign-loops=1 -fomit-frame-pointer"
%endif
%if %{_arch} == "x86_64"
OPTFLAGS="-O2 -fno-asynchronous-unwind-tables"
%endif
%if %{_arch} == "arm"
OPTFLAGS="-O2 -march=%{ARM_ARCH} %{ARM_FPU} -munaligned-access"
%endif
%endif

%if %{_os} == "mingw"
CFLAGS="$OPTFLAGS"
CXXFLAGS="$OPTFLAGS"
LDFLAGS=" "
%else
CFLAGS="$OPTFLAGS -ffunction-sections -fdata-sections"
CXXFLAGS="$OPTFLAGS"
LDFLAGS="-Wl,--gc-sections"
%endif

%if 1
cd libiberty
%configure \
	--build=%{build} \
	--host=%{host} \
	--target=%{target} \
%if (%{_os} == "mingw") && !%{BUILD_LOCAL_CROSS}
	--enable-static \
	--disable-shared \
%else
	--disable-static \
	--enable-shared \
%endif
%if %{_CROSS}
	--with-cross-host=%{target} \
%endif
	--disable-install-libiberty \
	&>Fconfigure_log.txt
cd ..

cd bfd
%configure \
	--build=%{build} \
	--host=%{host} \
	--target=%{target} \
%if (%{_os} == "mingw") && !%{BUILD_LOCAL_CROSS}
	--enable-static \
	--disable-shared \
%else
	--disable-static \
	--enable-shared \
%endif
	--disable-nls \
	--disable-libtool-lock \
%if %{_arch} == "arm"
	--disable-64-bit-bfd \
%else
	--enable-64-bit-bfd \
%endif
	--with-separate-debug-dir=%{tsysdir}/usr/%{_tlib}/debug \
	&>Fconfigure_log.txt
cd ..

cd libiberty
make &>FMake_log.txt
cd ..
cd bfd
make &>FMake_log.txt
cd ..
%endif

%if 1
cd opcodes
%configure \
	--build=%{build} \
	--host=%{host} \
	--target=%{target} \
%if (%{_os} == "mingw") && !%{BUILD_LOCAL_CROSS}
	--enable-static \
	--disable-shared \
%else
	--disable-static \
	--enable-shared \
%endif
	--disable-nls \
	--disable-libtool-lock \
	&>Fconfigure_log.txt

make &>FMake_log.txt
cd ..
%endif

%if %{_os} != "mingw"
%if %{BUILD_LOCAL_CROSS}
if test ! -e %{_exec_prefix}/%{_build}/%{target}/%{_lib}/libbfd-%{version}.so -o \
	! -e %{_exec_prefix}/%{_build}/%{target}/%{_lib}/libopcodes-%{version}.so; then

  rm -rf $RPM_BUILD_ROOT
  mkdir -p $RPM_BUILD_ROOT
  (cd bfd; make DESTDIR=$RPM_BUILD_ROOT install &>FInstall_log.txt)
  (cd opcodes; make DESTDIR=$RPM_BUILD_ROOT install &>FInstall_log.txt)
  rm $RPM_BUILD_ROOT%{bt_libdir}/*.la

  cat <<_EOF
----- ERROR ----- ERROR ----- ERROR ----- ERROR ----- ERROR ----- ERROR -----

ERROR: no libbfd-2.22.so or libopcodes-2.22.so
	copy RPM_BUILD_ROOT/%{name}-%{version}-%{release}.%{_arch}%{_exec_prefix}
	to   %{_exec_prefix}

-----------------------------------------------------------------------------
_EOF
  exit 1
fi
%endif
%endif #!mingw

%if 1
cd binutils
%configure \
	--build=%{build} \
	--host=%{host} \
	--target=%{target} \
%if (%{_os} == "mingw") && !%{BUILD_LOCAL_CROSS}
	--enable-static \
	--disable-shared \
%else
	--disable-static \
	--enable-shared \
%endif
%if %{_arch} == "arm"
	--disable-libquadmath \
	--disable-lto \
%endif
	--config-cache \
	--disable-libtool-lock \
	--disable-nls \
	&>Fconfigure_log.txt
make &>FMake_log.txt
cd ..
%endif

%if 1
cd gas
%configure \
	--build=%{build} \
	--host=%{host} \
	--target=%{target} \
%if (%{_os} == "mingw") && !%{BUILD_LOCAL_CROSS}
	--enable-static \
	--disable-shared \
%else
	--disable-static \
	--enable-shared \
%endif
	--disable-debug \
	--disable-libtool-lock \
	--disable-nls \
	&>Fconfigure_log.txt
make &>FMake_log.txt
cd ..
%endif

%if 1
cd ld
%configure \
	--build=%{build} \
	--host=%{host} \
	--target=%{target} \
%if %{_arch} == "arm"
	--disable-64-bit-bfd \
%else
	--enable-64-bit-bfd \
%endif
%if (%{_os} == "mingw") && !%{BUILD_LOCAL_CROSS}
	--enable-static \
	--disable-shared \
%else
	--disable-static \
	--enable-shared \
%endif
	--disable-multilib \
	--disable-libtool-lock \
	--disable-nls \
	--with-lib-path=%{tsysdir}/usr/%{_tlib}:%{tsysdir}/%{_tlib}:%{tsysdir}/usr/X11/%{_tlib}:%{tsysdir}/usr/local/%{_tlib} \
	&>Fconfigure_log.txt
make &>FMake_log.txt
cd ..
%endif

%if 1
cd gprof
%configure \
	--build=%{build} \
	--host=%{host} \
	--target=%{target} \
%if (%{_os} == "mingw") && !%{BUILD_LOCAL_CROSS}
	--disable-shared \
%else
	--enable-shared \
%endif
	--disable-libtool-lock \
	--disable-nls \
	&>Fconfigure_log.txt
make &>FMake_log.txt
cd ..
%endif

# ------------------- Install ----------------------------------------
%Install
%if %{_CROSS}
PATH=%{csysbindir}:$PATH:/opt/perl/bin
GCC_NoHostPath=d
export PATH GCC_NoHostPath
%else
export PATH=$PATH:/opt/perl/bin
%endif

rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT
mkdir -p %{_srcrpmdir}
mkdir -p $RPM_BUILD_ROOT%{docdir}

cd bfd; 
make DESTDIR=$RPM_BUILD_ROOT install &>FInstall_log.txt
cp bfdver.h $RPM_BUILD_ROOT%{bt_includedir}/
cd ..

(cd opcodes; make DESTDIR=$RPM_BUILD_ROOT install &>FInstall_log.txt)
(cd binutils; make DESTDIR=$RPM_BUILD_ROOT install &>FInstall_log.txt)
(cd gas; make DESTDIR=$RPM_BUILD_ROOT install &>FInstall_log.txt)
(cd ld; make DESTDIR=$RPM_BUILD_ROOT install &>FInstall_log.txt)
(cd gprof; make DESTDIR=$RPM_BUILD_ROOT install &>FInstall_log.txt)


rm -rf $RPM_BUILD_ROOT%{_infodir}/dir

cp COPYING COPYING.LIB COPYING3 COPYING3.LIB MAINTAINERS README $RPM_BUILD_ROOT%{docdir}/

rm $RPM_BUILD_ROOT%{bt_libdir}/*.la
rm -r $RPM_BUILD_ROOT%{t_prefix}/bin

%if %{BUILD_LOCAL_CROSS}
gzip -9 -n  $RPM_BUILD_ROOT%{_infodir}/*
rm -r $RPM_BUILD_ROOT%{_mandir}
%endif

%if %{_os} != "mingw"

pushd $RPM_BUILD_ROOT%{_bindir}
%if !%{BUILD_LOCAL_CROSS}
  for i in %{target}-* ; do
    mv $i ${i#%{target}-}
  done
%endif

%if %{BUILD_LOCAL_CROSS}
  rm %{target}-ld.bfd
  ln -s %{target}-ld %{target}-ld.bfd
%else
  rm ld.bfd
  ln -s ld ld.bfd
%endif
popd
%endif #!mingw

%if !%{_CROSS}
if test -f /opt/aldccp/bin/packages_idx; then
  /opt/aldccp/bin/packages_idx -p %{name} -v %{version} $RPM_BUILD_ROOT
fi
%endif

# ------------------- Clean ------------------------------------------
%Clean 

%if %{BUILD_LOCAL_CROSS}
  DE=%{version}-%{release}.%{_build_cpu}.rpm
  DD=%{_rpmdir}/%{_build_cpu}/%{ALD_Ver}%{CrossDestDir}/%{TO_ARCH}%{DestDir}

%else  #%{BUILD_LOCAL_CROSS}
  DE=%{version}-%{release}.%{TO_ARCH}.rpm
  DD=%{_rpmdir}/%{TO_ARCH}/%{ALD_Ver}%{CrossDestDir}%{DestDir}
%endif #!%{BUILD_LOCAL_CROSS}

pushd %{_rpmdir}/%{_target_cpu}
  SE=%{version}-%{release}.%{_target_cpu}.rpm
%if (%{_os} == "mingw") && !%{BUILD_LOCAL_CROSS}
  rm %{name}-$SE
%else  #mingw
  mkdir -p $DD
  mv %{name}-$SE $DD/%{name}-$DE
  mv %{name}-info-$SE $DD/%{name}-info-$DE
  mv %{name}-devel-$SE $DD/%{name}-devel-$DE
  mv %{name}-ldscripts-$SE $DD/%{name}-ldscripts-$DE
%endif #!mingw
popd

# --------------- build *.tgz ------------------------
%if %{BUILD_TAR}
pushd $RPM_BUILD_ROOT
%if (%{_os} == "mingw") && !%{BUILD_LOCAL_CROSS}
  TMP_TO=$DD
%else
  TMP_TO=$DD/tar
%endif
  mkdir -p $TMP_TO
  tmp=%{name}-%{version}-%{release}
  mkdir -p $tmp
%if %{BUILD_LOCAL_CROSS}
  cp -r cross $tmp/
%else
  cp -r usr $tmp/
%endif
%if (%{_os} == "mingw") && !%{BUILD_LOCAL_CROSS}
  zip -9 -r $tmp.zip $tmp >/dev/null
  mv $tmp.zip $TMP_TO
%else
  tar -cjf $tmp.tar.bz2 $tmp
  mv $tmp.tar.bz2 $TMP_TO
%endif
  rm -r $tmp
popd
%endif  #BUILD_TAR

# ------------------- Files ------------------------------------------
%if (%{_os} == "mingw") && !%{BUILD_LOCAL_CROSS}
%Files
%{_prefix}

%else

%Files
%{_bindir}
%{bt_libdir}/libbfd-%{version}.so
%{bt_libdir}/libopcodes-%{version}.so
%if !%{BUILD_LOCAL_CROSS}
%{_mandir}
%endif
%{docdir}

%files info
%{_infodir}

%files devel
%{bt_includedir}
%{bt_libdir}/libbfd.so
%{bt_libdir}/libopcodes.so

%files ldscripts
%{t_prefix}/lib/ldscripts

%endif #!mingw
